@@ -13,165 +13,15 @@ from sapiopycommons.general.exceptions import SapioUserErrorException
 
 
 # FR-46064 - Initial port of PyWebhookUtils to sapiopycommons.
+# FR-46716 - Add comments noting that some functions are deprecated in favor of CallbackUtil.
 class FileUtil:
     """
     Utilities for the handling of files, including the requesting of files from the user and the parsing of files into
     tokenized lists. Makes use of Pandas DataFrames for any file parsing purposes.
     """
-    # FR-46097 - Add write file request shorthand functions to FileUtil.
     @staticmethod
-    def write_file(file_name: str, file_bytes: bytes, *, request_context: str | None = None) -> SapioWebhookResult:
-        """
-        Send a file to the client.
-
-        The calling webhook must catch the WriteFileResult that the client will send back.
-
-        :param file_name: The name of the file.
-        :param file_bytes: The bytes of the file.
-        :param request_context: Context that will be returned to the webhook server in the client callback result.
-        :return: A SapioWebhookResult with the write request as its client callback request.
-        """
-        return SapioWebhookResult(True, client_callback_request=WriteFileRequest(file_bytes, file_name,
-                                                                                 request_context))
-
-    @staticmethod
-    def write_files(files: dict[str, bytes], *, request_context: str | None = None) -> SapioWebhookResult:
-        """
-        Send a collection of files to the client.
-
-        The calling webhook must catch the MultiFileResult that the client will send back.
-
-        :param files: A dictionary of files names to file bytes.
-        :param request_context: Context that will be returned to the webhook server in the client callback result.
-        :return: A SapioWebhookResult with the write request as its client callback request.
-        """
-        return SapioWebhookResult(True, client_callback_request=MultiFileRequest(files, request_context))
-
-    @staticmethod
-    def request_file(context: SapioWebhookContext, title: str, exts: list[str] = None,
-                     *, request_context: str | None = None) \
-            -> tuple[SapioWebhookResult | None, str | None, bytes | None]:
-        """
-        Request a single file from the user. This function handles the entire client callback interaction for the
-        requesting of the file, including if the user cancels the file upload prompt.
-
-        The first time this method is called in the course of an interaction with the client, it will return a webhook
-        result containing the client callback to request the file. The second time it is called, it will return the
-        file name and bytes from the callback result.
-
-        :param context: The current webhook context.
-        :param title: The title of the file prompt dialog.
-        :param exts: The allowable file extensions of the uploaded file. If blank, any file can be uploaded. Throws an
-            exception if an incorrect file extension is provided.
-        :param request_context: Context that will be returned to the webhook server in the client callback result.
-        :return: A tuple with the following elements.
-            0 - A webhook result that contains a file prompt if this is the first interaction with this request.
-            May also contain a result that will terminate the client interaction if the user canceled the prompt.
-            1 - The file name of the requested file if the user provided one.
-            2 - The file bytes of the requested file if the user provided one.
-        """
-        client_callback = context.client_callback_result
-        result_context: str | None = client_callback.callback_context_data if client_callback else None
-        # If the user cancels, terminate the interaction.
-        if client_callback is not None and client_callback.user_cancelled:
-            return SapioWebhookResult(True), None, None
-        # If no extensions were provided, use an empty list for the extensions instead.
-        if exts is None:
-            exts = []
-
-        # If the client callback isn't a FilePromptResult, then it's either None or some other callback result, meaning
-        # we need to send a new request. We may also send a new request if the client callback result is a
-        # FilePromptResult, but its callback context doesn't match the provided callback context, meaning it's a
-        # result from a different call to request_file.
-        is_file_result = isinstance(client_callback, FilePromptResult)
-        if not is_file_result or (is_file_result and result_context != request_context):
-            prompt = FilePromptRequest(dialog_title=title, file_extension=",".join(exts),
-                                       callback_context_data=request_context)
-            return SapioWebhookResult(True, client_callback_request=prompt), None, None
-
-        # Get the file from the result. Enforce that the provided data isn't empty, and that the file path ends in
-        # one of the allowed extensions.
-        # noinspection PyTypeChecker
-        result: FilePromptResult = client_callback
-        file_path: str | None = result.file_path
-        file_bytes: bytes | None = result.file_bytes
-        FileUtil.__verify_file(file_path, file_bytes, exts)
-        return None, file_path, file_bytes
-
-    @staticmethod
-    def request_files(context: SapioWebhookContext, title: str, exts: list[str] = None,
-                      *, request_context: str | None = None) \
-            -> tuple[SapioWebhookResult | None, dict[str, bytes] | None]:
-        """
-        Request multiple files from the user. This function handles the entire client callback interaction for the
-        requesting of the files, including if the user cancels the file upload prompt.
-
-        The first time this method is called in the course of an interaction with the client, it will return a webhook
-        result containing the client callback to request the files. The second time it is called, it will return each
-        file name and bytes from the callback result.
-
-        :param context: The current webhook context.
-        :param title: The title of the file prompt dialog.
-        :param exts: The allowable file extensions of the uploaded file. If blank, any file can be uploaded. Throws an
-            exception if an incorrect file extension is provided.
-        :param request_context: Context that will be returned to the webhook server in the client callback result.
-        :return: A tuple with the following elements.
-            0 - A webhook result that contains a file prompt if this is the first interaction with this request.
-            May also contain a result that will terminate the client interaction if the user canceled the prompt.
-            1 - A dictionary that maps the file names to the file bytes for each provided file.
-        """
-        client_callback = context.client_callback_result
-        result_context: str | None = client_callback.callback_context_data if client_callback else None
-        # If the user cancels, terminate the interaction.
-        if client_callback is not None and client_callback.user_cancelled:
-            return SapioWebhookResult(True), None
-        # If no extensions were provided, use an empty list for the extensions instead.
-        if exts is None:
-            exts = []
-
-        # If the client callback isn't a MultiFilePromptResult, then it's either None or some other callback result,
-        # meaning we need to send a new request. We may also send a new request if the client callback result is a
-        # MultiFilePromptResult, but its callback context doesn't match the provided callback context, meaning it's a
-        # result from a different call to request_file.
-        is_file_result = isinstance(client_callback, MultiFilePromptResult)
-        if not is_file_result or (is_file_result and result_context != request_context):
-            prompt = MultiFilePromptRequest(dialog_title=title, file_extension=",".join(exts),
-                                            callback_context_data=request_context)
-            return SapioWebhookResult(True, client_callback_request=prompt), None
-
-        # Get the files from the result. Enforce that the provided data isn't empty, and that the file paths end in
-        # one of the allowed extensions.
-        # noinspection PyTypeChecker
-        result: MultiFilePromptResult = client_callback
-        for file_path, file_bytes in result.files.items():
-            FileUtil.__verify_file(file_path, file_bytes, exts)
-        return None, result.files
-
-    @staticmethod
-    def __verify_file(file_path: str, file_bytes: bytes, allowed_extensions: list[str]):
-        """
-        Verify that the provided file was read (i.e. the file path and file bytes aren't None or empty) and that it
-        has the correct file extension. Raises a user error exception if something about the file is incorrect.
-
-        :param file_path: The name of the file to verify.
-        :param file_bytes: The bytes of the file to verify.
-        :param allowed_extensions: The file extensions that the file path is allowed to have.
-        """
-        if file_path is None or len(file_path) == 0 or file_bytes is None or len(file_bytes) == 0:
-            raise SapioUserErrorException("Empty file provided or file unable to be read.")
-        if len(allowed_extensions) != 0:
-            matches: bool = False
-            for ext in allowed_extensions:
-                if file_path.endswith("." + ext):
-                    matches = True
-                    break
-            if matches is False:
-                raise SapioUserErrorException("Unsupported file type. Expecting the following extension(s): "
-                                              + (",".join(allowed_extensions)))
-
-    @staticmethod
-    def tokenize_csv(file_bytes: bytes, required_headers: list[str] | None = None, header_row_index: int | None = 0) \
-            -> tuple[list[dict[str, str]], list[list[str]]]:
+    def tokenize_csv(file_bytes: bytes, required_headers: list[str] | None = None, header_row_index: int | None = 0,
+                     seperator: str = ",") -> tuple[list[dict[str, str]], list[list[str]]]:
         """
         Tokenize a CSV file. The provided file must be uniform. That is, if row 1 has 10 cells, all the rows in the file
         must have 10 cells. Otherwise, the Pandas parser throws a tokenizer exception.
@@ -183,13 +33,14 @@ class FileUtil:
             row is returned in the metadata list. If input is None, then no row is considered to be the header row,
             meaning that required headers are also ignored if any are provided. By default, the first row (0th index)
             is assumed to be the header row.
+        :param seperator: The character that separates cells in the table.
         :return: The CSV parsed into a list of dicts where each dict is a row, mapping the headers to the cells for
             that row. Also returns a list of each row above the headers (the metadata), parsed into a list of each cell.
             If the header row index is 0 or None, this list will be empty.
         """
         # Parse the file bytes into two DataFrames. The first is metadata of the file located above the header row,
         # while the second is the body of the file below the header row.
-        file_body, file_metadata = FileUtil.csv_to_data_frames(file_bytes, header_row_index)
+        file_body, file_metadata = FileUtil.csv_to_data_frames(file_bytes, header_row_index, seperator)
         # Parse the metadata from above the header row index into a list of lists.
         metadata: list[list[str]] = FileUtil.data_frame_to_lists(file_metadata)
         # Parse the data from the file body into a list of dicts.
@@ -223,7 +74,7 @@ class FileUtil:
         return rows, metadata
 
     @staticmethod
-    def csv_to_data_frames(file_bytes: bytes, header_row_index: int | None = 0) \
+    def csv_to_data_frames(file_bytes: bytes, header_row_index: int | None = 0, seperator: str = ",") \
             -> tuple[DataFrame, DataFrame | None]:
         """
         Parse the file bytes for a CSV into DataFrames. The provided file must be uniform. That is, if row 1 has 10
@@ -234,6 +85,7 @@ class FileUtil:
             row is returned in the metadata list. If input is None, then no row is considered to be the header row,
             meaning that required headers are also ignored if any are provided. By default, the first row (0th index)
             is assumed to be the header row.
+        :param seperator: The character that separates cells in the table.
         :return: A tuple of two DataFrames. The first is the frame for the CSV table body, while the second is for the
             metadata from above the header row, or None if there is no metadata.
         """
@@ -245,13 +97,13 @@ class FileUtil:
                 # can throw off the header row index.
                 file_metadata = pandas.read_csv(file_io, header=None, dtype=dtype(str),
                                                 skiprows=lambda x: x >= header_row_index,
-                                                skip_blank_lines=False)
+                                                skip_blank_lines=False, sep=seperator)
         with io.BytesIO(file_bytes) as file_io:
             # The use of the dtype argument is to ensure that everything from the file gets read as a string. Added
             # because some numerical values would get ".0" appended to them, even when casting the DataFrame cell to a
             # string.
             file_body: DataFrame = pandas.read_csv(file_io, header=header_row_index, dtype=dtype(str),
-                                                   skip_blank_lines=False)
+                                                   skip_blank_lines=False, sep=seperator)
 
         return file_body, file_metadata
 
@@ -380,3 +232,197 @@ class FileUtil:
                 data_frame.to_excel(writer, sheet_name='Sheet1', header=False, index=False)
             xlsx_data = output.getvalue()
         return xlsx_data
+
+    @staticmethod
+    def csv_to_xls(file_data: bytes | str, delimiter: str = ",", newline: str = "\r\n") -> bytes:
+        """
+        Convert the bytes or string of a .csv file to .xls bytes.
+
+        :param file_data: The .csv bytes or string to convert.
+        :param delimiter: The delimiter character separating columns, with "," being the default.
+        :param newline: The newline character(s) separating rows, with "\r\n" being the default.
+        :return: The bytes of the new .xls file.
+        """
+        # Import the libraries we'll need locally since we won't be using them anywhere else in the class.
+        from xlwt import Workbook, Worksheet
+
+        # Create an Excel workbook along with a worksheet, which is where the data will be written.
+        workbook: Workbook = Workbook()
+        sheet: Worksheet = workbook.add_sheet("Sheet1")
+
+        # Make sure the file data is in a string format so that we can work with it.
+        formatted_data: str = bytes.decode(file_data, "utf-8") if isinstance(file_data, bytes) else file_data
+
+        # Write each row of the file to the .xls sheet.
+        rows: list[str] = formatted_data.split(newline)
+        for i, row in enumerate(rows):
+            values: list[str] = row.split(delimiter)
+            for j, value in enumerate(values):
+                sheet.write(i, j, value)
+
+        # Save the worksheet data to the byte buffer and return the bytes.
+        with io.BytesIO() as buffer:
+            workbook.save(buffer)
+            file_bytes: bytes = buffer.getvalue()
+        return file_bytes
+
+    # Deprecated functions:
+
+    # FR-46097 - Add write file request shorthand functions to FileUtil.
+    @staticmethod
+    def write_file(file_name: str, file_bytes: bytes, *, request_context: str | None = None) -> SapioWebhookResult:
+        """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
+        Send a file to the client.
+
+        The calling webhook must catch the WriteFileResult that the client will send back.
+
+        :param file_name: The name of the file.
+        :param file_bytes: The bytes of the file.
+        :param request_context: Context that will be returned to the webhook server in the client callback result.
+        :return: A SapioWebhookResult with the write request as its client callback request.
+        """
+        return SapioWebhookResult(True, client_callback_request=WriteFileRequest(file_bytes, file_name,
+                                                                                 request_context))
+
+    @staticmethod
+    def write_files(files: dict[str, bytes], *, request_context: str | None = None) -> SapioWebhookResult:
+        """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
+        Send a collection of files to the client.
+
+        The calling webhook must catch the MultiFileResult that the client will send back.
+
+        :param files: A dictionary of files names to file bytes.
+        :param request_context: Context that will be returned to the webhook server in the client callback result.
+        :return: A SapioWebhookResult with the write request as its client callback request.
+        """
+        return SapioWebhookResult(True, client_callback_request=MultiFileRequest(files, request_context))
+
+    @staticmethod
+    def request_file(context: SapioWebhookContext, title: str, exts: list[str] = None,
+                     *, request_context: str | None = None) \
+            -> tuple[SapioWebhookResult | None, str | None, bytes | None]:
+        """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
+        Request a single file from the user. This function handles the entire client callback interaction for the
+        requesting of the file, including if the user cancels the file upload prompt.
+
+        The first time this method is called in the course of an interaction with the client, it will return a webhook
+        result containing the client callback to request the file. The second time it is called, it will return the
+        file name and bytes from the callback result.
+
+        :param context: The current webhook context.
+        :param title: The title of the file prompt dialog.
+        :param exts: The allowable file extensions of the uploaded file. If blank, any file can be uploaded. Throws an
+            exception if an incorrect file extension is provided.
+        :param request_context: Context that will be returned to the webhook server in the client callback result.
+        :return: A tuple with the following elements.
+            0 - A webhook result that contains a file prompt if this is the first interaction with this request.
+            May also contain a result that will terminate the client interaction if the user canceled the prompt.
+            1 - The file name of the requested file if the user provided one.
+            2 - The file bytes of the requested file if the user provided one.
+        """
+        client_callback = context.client_callback_result
+        result_context: str | None = client_callback.callback_context_data if client_callback else None
+        # If the user cancels, terminate the interaction.
+        if client_callback is not None and client_callback.user_cancelled:
+            return SapioWebhookResult(True), None, None
+        # If no extensions were provided, use an empty list for the extensions instead.
+        if exts is None:
+            exts = []
+
+        # If the client callback isn't a FilePromptResult, then it's either None or some other callback result, meaning
+        # we need to send a new request. We may also send a new request if the client callback result is a
+        # FilePromptResult, but its callback context doesn't match the provided callback context, meaning it's a
+        # result from a different call to request_file.
+        is_file_result = isinstance(client_callback, FilePromptResult)
+        if not is_file_result or (is_file_result and result_context != request_context):
+            prompt = FilePromptRequest(dialog_title=title, file_extension=",".join(exts),
+                                       callback_context_data=request_context)
+            return SapioWebhookResult(True, client_callback_request=prompt), None, None
+
+        # Get the file from the result. Enforce that the provided data isn't empty, and that the file path ends in
+        # one of the allowed extensions.
+        # noinspection PyTypeChecker
+        result: FilePromptResult = client_callback
+        file_path: str | None = result.file_path
+        file_bytes: bytes | None = result.file_bytes
+        FileUtil.__verify_file(file_path, file_bytes, exts)
+        return None, file_path, file_bytes
+
+    @staticmethod
+    def request_files(context: SapioWebhookContext, title: str, exts: list[str] = None,
+                      *, request_context: str | None = None) \
+            -> tuple[SapioWebhookResult | None, dict[str, bytes] | None]:
+        """
+        DEPRECATED: Make use of CallbackUtil as of 24.5.
+
+        Request multiple files from the user. This function handles the entire client callback interaction for the
+        requesting of the files, including if the user cancels the file upload prompt.
+
+        The first time this method is called in the course of an interaction with the client, it will return a webhook
+        result containing the client callback to request the files. The second time it is called, it will return each
+        file name and bytes from the callback result.
+
+        :param context: The current webhook context.
+        :param title: The title of the file prompt dialog.
+        :param exts: The allowable file extensions of the uploaded file. If blank, any file can be uploaded. Throws an
+            exception if an incorrect file extension is provided.
+        :param request_context: Context that will be returned to the webhook server in the client callback result.
+        :return: A tuple with the following elements.
+            0 - A webhook result that contains a file prompt if this is the first interaction with this request.
+            May also contain a result that will terminate the client interaction if the user canceled the prompt.
+            1 - A dictionary that maps the file names to the file bytes for each provided file.
+        """
+        client_callback = context.client_callback_result
+        result_context: str | None = client_callback.callback_context_data if client_callback else None
+        # If the user cancels, terminate the interaction.
+        if client_callback is not None and client_callback.user_cancelled:
+            return SapioWebhookResult(True), None
+        # If no extensions were provided, use an empty list for the extensions instead.
+        if exts is None:
+            exts = []
+
+        # If the client callback isn't a MultiFilePromptResult, then it's either None or some other callback result,
+        # meaning we need to send a new request. We may also send a new request if the client callback result is a
+        # MultiFilePromptResult, but its callback context doesn't match the provided callback context, meaning it's a
+        # result from a different call to request_file.
+        is_file_result = isinstance(client_callback, MultiFilePromptResult)
+        if not is_file_result or (is_file_result and result_context != request_context):
+            prompt = MultiFilePromptRequest(dialog_title=title, file_extension=",".join(exts),
+                                            callback_context_data=request_context)
+            return SapioWebhookResult(True, client_callback_request=prompt), None
+
+        # Get the files from the result. Enforce that the provided data isn't empty, and that the file paths end in
+        # one of the allowed extensions.
+        # noinspection PyTypeChecker
+        result: MultiFilePromptResult = client_callback
+        for file_path, file_bytes in result.files.items():
+            FileUtil.__verify_file(file_path, file_bytes, exts)
+        return None, result.files
+
+    @staticmethod
+    def __verify_file(file_path: str, file_bytes: bytes, allowed_extensions: list[str]):
+        """
+        Verify that the provided file was read (i.e. the file path and file bytes aren't None or empty) and that it
+        has the correct file extension. Raises a user error exception if something about the file is incorrect.
+
+        :param file_path: The name of the file to verify.
+        :param file_bytes: The bytes of the file to verify.
+        :param allowed_extensions: The file extensions that the file path is allowed to have.
+        """
+        if file_path is None or len(file_path) == 0 or file_bytes is None or len(file_bytes) == 0:
+            raise SapioUserErrorException("Empty file provided or file unable to be read.")
+        if len(allowed_extensions) != 0:
+            matches: bool = False
+            for ext in allowed_extensions:
+                if file_path.endswith("." + ext):
+                    matches = True
+                    break
+            if matches is False:
+                raise SapioUserErrorException("Unsupported file type. Expecting the following extension(s): "
+                                              + (",".join(allowed_extensions)))
